package com.ltt.compass.weather.api

import android.util.Log
import com.google.gson.Gson
import okhttp3.MediaType
import okhttp3.OkHttpClient
import okhttp3.RequestBody
import okhttp3.ResponseBody
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import java.lang.reflect.ParameterizedType
import java.util.concurrent.TimeUnit
import java.util.logging.Level

/**
 * Retrofit请求工具类
 */
open class RetrofitRequest private constructor() {
    private val TIME_OUT = 30 // 30秒超时断开连接
    private val gson = Gson()
    private var mRetrofit: Retrofit? = null
    /**
     * ERROR
     */
    private var BASE_URL = ""

    /**
     * 单例
     */
    companion object {
        var DEBUG = true
        @Volatile
        private var sRetrofitRequest: RetrofitRequest? = null
        const val CODE_NEW_ERROR = 700//网络失败
        const val CODE_SERVICE_ERROR = 80//服务器失败
        const val CODE_DATA_ERROR = 900//解释或者其他报错
        fun getInstance(): RetrofitRequest {
            if (sRetrofitRequest == null) {
                synchronized(RetrofitRequest::class.java) {
                    if (sRetrofitRequest == null) {
                        sRetrofitRequest = RetrofitRequest()
                    }
                }
            }
            return sRetrofitRequest!!
        }
    }

    open fun changeApiBaseUrl(baseUrl: String) {
        this.BASE_URL = baseUrl
    }

    open fun setApiBaseUrl(baseUrl: String): RetrofitRequest {
        this.BASE_URL = baseUrl
        mRetrofit = createRetrofit()
        return this
    }

    private fun createRetrofit(): Retrofit {
        val builder = OkHttpClient.Builder()
        builder.connectTimeout(TIME_OUT.toLong(), TimeUnit.SECONDS)
        builder.readTimeout(TIME_OUT.toLong(), TimeUnit.SECONDS)
        builder.writeTimeout(TIME_OUT.toLong(), TimeUnit.SECONDS)
        builder.retryOnConnectionFailure(true)
        if (DEBUG) {
            val loggingInterceptor = LogInterceptor("RetrofitRequest")
            loggingInterceptor.setPrintLevel(LogInterceptor.Level.BODY)
            loggingInterceptor.setColorLevel(Level.INFO)
            builder.addInterceptor(loggingInterceptor)
        }
        return Retrofit.Builder()
                .client(builder.build())
                .addConverterFactory(GsonConverterFactory.create())
                .baseUrl(BASE_URL)
                .build()
    }

    /**
     * 发送GET网络请求
     * @param url           请求地址
     * @param clazz         返回的数据类型
     * @param resultHandler 回调
     * @param <T>           泛型
     */
    fun <T> sendGetRequest(url: String, resultCallBack: ResultCallBack<T>) { // 判断网络连接状况
        if (mRetrofit == null) {
            Log.e("RetrofitRequest", "未调用setApiBaseUrl()")
            return
        }
        resultCallBack.onReadyResult()
        val getRequest = mRetrofit!!.create(GetRequest::class.java)
        // 构建请求
        val call = getRequest.getUrl(url)
        call!!.enqueue(object : Callback<ResponseBody?> {
            override fun onResponse(call: Call<ResponseBody?>, response: Response<ResponseBody?>) {
                    val body = response.body()
                    if (body == null) {
                        resultCallBack.onFailure(CODE_SERVICE_ERROR, "服务器繁忙")
                        return
                    }
                    val jsonStr = body.string()
                    getGeneric(jsonStr, resultCallBack)
            }

            override fun onFailure(call: Call<ResponseBody?>, t: Throwable) {
                if (t.cause != null) {
                    if (t.cause!!.message != null) {
                        resultCallBack.onFailure(CODE_NEW_ERROR, t.cause!!.message!!)
                    } else {
                        if (t.message != null) {
                            resultCallBack.onFailure(CODE_NEW_ERROR, t.message!!)
                        }else{
                            resultCallBack.onFailure(CODE_NEW_ERROR, "服务器繁忙")
                        }
                    }
                }
            }
        })
    }

    /**
     * 发送post网络请求
     * @param url           请求地址
     * @param paramMap      参数列表
     * @param clazz         返回的数据类型
     * @param resultCallBack 回调
     * @param <T>           泛型
     */
    fun <T> sendPostMapRequest(url: String, paramMap: HashMap<String, String>, resultCallBack: ResultCallBack<T>) { // 判断网络连接状况
        if (mRetrofit == null) {
            Log.e("RetrofitRequest", "未调用setApiBaseUrl()")
            return
        }
        resultCallBack.onReadyResult()
        val postRequest = mRetrofit!!.create(PostRequest::class.java)
        // 构建请求
        val call = postRequest.postMap(url, paramMap)
        call.enqueue(object : Callback<ResponseBody?> {
            override fun onResponse(call: Call<ResponseBody?>, response: Response<ResponseBody?>) {
                    val body = response.body()
                    if (body == null) {
                        resultCallBack.onFailure(CODE_SERVICE_ERROR, "服务器繁忙")
                        return
                    }
                    val jsonStr = body.string()
                    getGeneric(jsonStr, resultCallBack)
            }

            override fun onFailure(call: Call<ResponseBody?>, t: Throwable) {
                if (t.cause != null) {
                    if (t.cause!!.message != null) {
                        resultCallBack.onFailure(CODE_NEW_ERROR, t.cause!!.message!!)
                    } else {
                        if (t.message != null) {
                            resultCallBack.onFailure(CODE_NEW_ERROR, t.message!!)
                        }else{
                            resultCallBack.onFailure(CODE_NEW_ERROR, "服务器繁忙")
                        }
                    }
                }
            }
        })
    }


    /**
     * 发送post网络请求
     * @param url           请求地址
     * @param json           json参数
     * @param clazz         返回的数据类型
     * @param resultCallBack 回调
     * @param <T>           泛型
     */
    fun <T> sendPostJson(url: String, json: String, resultCallBack: ResultCallBack<T>) {
        if (mRetrofit == null) {
            Log.e("RetrofitRequest", "未调用setApiBaseUrl()")
            return
        }
        resultCallBack.onReadyResult()
        val postRequest = mRetrofit!!.create(PostRequest::class.java)
        // 构建请求
        val call = postRequest.postJson(url, RequestBody.create(MediaType.parse("application/json;charset=UTF-8"), json))
        call.enqueue(object : Callback<ResponseBody?> {
            override fun onResponse(call: Call<ResponseBody?>, response: Response<ResponseBody?>) {
                    val body = response.body()
                    if (body == null) {
                        resultCallBack.onFailure(CODE_SERVICE_ERROR, "服务器繁忙")
                        return
                    }
                    val jsonStr = body.string()
                    getGeneric(jsonStr, resultCallBack)
            }

            override fun onFailure(call: Call<ResponseBody?>, t: Throwable) {
                if (t.cause != null) {
                    if (t.cause!!.message != null) {
                        resultCallBack.onFailure(CODE_NEW_ERROR, t.cause!!.message!!)
                    } else {
                        if (t.message != null) {
                            resultCallBack.onFailure(CODE_NEW_ERROR, t.message!!)
                        }else{
                            resultCallBack.onFailure(CODE_NEW_ERROR, "服务器繁忙")
                        }
                    }
                }
            }
        })
    }


    /**
     * 自动解析json至回调中的JavaBean
     */
    private fun <T> getGeneric(jsonData: String, callBack: ResultCallBack<T>) {
        try {
            //通过反射获取泛型的实例
            val types = callBack.javaClass.genericInterfaces
            val actualTypeArguments = (types[0] as ParameterizedType).actualTypeArguments
            val type = actualTypeArguments[0]
            callBack.onSuccessJson(jsonData)
            if ("class java.lang.String" != type.toString()) {
                callBack.onSuccess(gson.fromJson(jsonData, type))
            }
        } catch (e: Exception) {
            e.printStackTrace()
            callBack.onFailure(CODE_DATA_ERROR, "数据解析错误")
        }
    }
}